//
//  ErrorBarsChartView.m
//  SciChartDemo
//
//  Created by Hrybenuik Mykola on 9/17/16.
//  Copyright © 2016 ABT. All rights reserved.
//

#import "ErrorBarsChartView.h"
#import <SciChart/SciChart.h>
#import "DataManager.h"

@implementation ErrorBarsChartView

@synthesize sciChartSurfaceView;
@synthesize surface;

- (void)initializeSurfaceRenderableSeries {
    
    int dataCount = 10;
    SCIXyDataSeries * priceDataSeries = [[SCIXyDataSeries alloc] initWithXType:SCIDataType_Float YType:SCIDataType_Float];
    //Getting Fourier dataSeries
    for (int i = 0; i < dataCount; i++) {
        double time = 10 * i / (double)dataCount;
        double x = time;
        double y = randf(-1.5, 1.5);
        [priceDataSeries appendX:SCIGeneric(x) Y:SCIGeneric(y)];
    }
    
    priceDataSeries.dataDistributionCalculator = [SCIUserDefinedDistributionCalculator new];
    
    SCIFastFixedErrorBarsRenderableSeries *verticalRenderableSeries = [SCIFastFixedErrorBarsRenderableSeries new];
    verticalRenderableSeries.errorType = SCIErrorBarTypeRelative;
    verticalRenderableSeries.errorLow = 0.1;
    verticalRenderableSeries.errorHigh = 0.3;
    verticalRenderableSeries.style.linePen = [[SCIPenSolid alloc] initWithColor:[UIColor colorWithRed:70.f/255.f
                                                                                                green:130.f/255.f
                                                                                                 blue:180.f/255.f
                                                                                                alpha:1.f]
                                                                          Width:1.f];
    verticalRenderableSeries.xAxisId = @"xAxis";
    verticalRenderableSeries.yAxisId = @"yAxis";
    [verticalRenderableSeries setDataSeries:priceDataSeries];
    [surface attachRenderableSeries:verticalRenderableSeries];
    
    SCIFastFixedErrorBarsRenderableSeries *horizontalRenderableSeries = [SCIFastFixedErrorBarsRenderableSeries new];
    horizontalRenderableSeries.style.linePen = [[SCIPenSolid alloc] initWithColor:[UIColor redColor] Width:1.f];
    horizontalRenderableSeries.errorDirection = SCIErrorBarDirectionHorizontal;
    horizontalRenderableSeries.errorDataPointWidth = 0.5f;
    horizontalRenderableSeries.xAxisId = @"xAxis";
    horizontalRenderableSeries.yAxisId = @"yAxis";
    horizontalRenderableSeries.errorMode = SCIErrorBarModeBoth;
    [horizontalRenderableSeries setDataSeries:priceDataSeries];
    [surface attachRenderableSeries:horizontalRenderableSeries];
    
    SCIEllipsePointMarker * ellipsePointMarker = [[SCIEllipsePointMarker alloc]init];
    [ellipsePointMarker setDrawBorder:YES];
    [ellipsePointMarker setFillBrush:[[SCIBrushSolid alloc] initWithColor:[UIColor colorWithRed:70.f/255.f
                                                                                          green:130.f/255.f
                                                                                           blue:180.f/255.f
                                                                                          alpha:1.f]]];
    [ellipsePointMarker setBorderPen:[[SCIPenSolid alloc] initWithColor:[UIColor colorWithRed:176.f/255.f
                                                                                        green:196.f/255.f
                                                                                         blue:222.f/255.f
                                                                                        alpha:1.f]
                                                                  Width:2.f]];
    [ellipsePointMarker setHeight:15];
    [ellipsePointMarker setWidth:15];
    
    SCIFastLineRenderableSeries *lineRenderableSeries = [SCIFastLineRenderableSeries new];
    lineRenderableSeries.style.drawPointMarkers = YES;
    lineRenderableSeries.style.pointMarker = ellipsePointMarker;
    lineRenderableSeries.style.linePen = [[SCIPenSolid alloc] initWithColor:[UIColor colorWithRed:176.f/255.f
                                                                                            green:196.f/255.f
                                                                                             blue:222.f/255.f
                                                                                            alpha:1.f]
                                                                      Width:1.f];
    lineRenderableSeries.xAxisId = @"xAxis";
    lineRenderableSeries.yAxisId = @"yAxis";
    [lineRenderableSeries setDataSeries:priceDataSeries];
    [surface attachRenderableSeries:lineRenderableSeries];
    
    [surface invalidateElement];
}

- (instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    
    if (self) {
        SCIChartSurfaceView * view = [[SCIChartSurfaceView alloc]initWithFrame:frame];
        sciChartSurfaceView = view;
        
        [sciChartSurfaceView setTranslatesAutoresizingMaskIntoConstraints:NO];
        
        [self addSubview:sciChartSurfaceView];
        NSDictionary *layout = @{@"SciChart":sciChartSurfaceView};
        
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        [self addConstraints:[NSLayoutConstraint constraintsWithVisualFormat:@"V:|-(0)-[SciChart]-(0)-|" options:0 metrics:0 views:layout]];
        
        [self setupSurface];
        [self addAxes];
        [self addModifiers];
        [self initializeSurfaceRenderableSeries];
    }
    
    return self;
}

- (void)setupSurface {
    surface = [[SCIChartSurface alloc] initWithView: sciChartSurfaceView];
    
    [[surface style] setBackgroundBrush: [[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
    [[surface style] setSeriesBackgroundBrush:[[SCIBrushSolid alloc] initWithColorCode:0xFF1e1c1c]];
}

- (void)addAxes{
    SCIPenSolid * majorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF393532 Width:0.5];
    SCIBrushSolid * gridBandPen = [[SCIBrushSolid alloc] initWithColorCode:0xE1232120];
    SCIPenSolid * minorPen = [[SCIPenSolid alloc] initWithColorCode:0xFF262423 Width:0.5];
    
    SCITextFormattingStyle *  textFormatting= [[SCITextFormattingStyle alloc] init];
    [textFormatting setFontSize:16];
    [textFormatting setFontName:@"Helvetica"];
    [textFormatting setColorCode:0xFFafb3b6];
    
    SCIAxisStyle * axisStyle = [[SCIAxisStyle alloc]init];
    [axisStyle setMajorTickBrush:majorPen];
    [axisStyle setGridBandBrush: gridBandPen];
    [axisStyle setMajorGridLineBrush:majorPen];
    [axisStyle setMinorTickBrush:minorPen];
    [axisStyle setMinorGridLineBrush:minorPen];
    [axisStyle setLabelStyle:textFormatting ];
    [axisStyle setDrawMinorGridLines:YES];
    [axisStyle setDrawMajorBands:YES];
    
    id<SCIAxis2D> axis = [[SCINumericAxis alloc] init];
    [axis setStyle: axisStyle];
    axis.axisId = @"yAxis";
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [surface attachAxis:axis IsXAxis:NO];
    
    axis = [[SCINumericAxis alloc] init];
    axis.axisId = @"xAxis";
    [axis setStyle: axisStyle];
    [axis setGrowBy: [[SCIDoubleRange alloc]initWithMin:SCIGeneric(0.1) Max:SCIGeneric(0.1)]];
    [surface attachAxis:axis IsXAxis:YES];
}

- (void)addModifiers{
    SCIXAxisDragModifier * xDragModifier = [SCIXAxisDragModifier new];
    xDragModifier.axisId = @"xAxis";
    xDragModifier.dragMode = SCIAxisDragMode_Scale;
    xDragModifier.clipModeX = SCIZoomPanClipMode_None;
    [xDragModifier setModifierName:@"XAxis DragModifier"];
    
    SCIYAxisDragModifier * yDragModifier = [SCIYAxisDragModifier new];
    yDragModifier.axisId = @"yAxis";
    yDragModifier.dragMode = SCIAxisDragMode_Pan;
    [yDragModifier setModifierName:@"YAxis DragModifier"];
    
    SCIPinchZoomModifier * pzm = [[SCIPinchZoomModifier alloc] init];
    [pzm setModifierName:@"PinchZoom Modifier"];
    
    SCIZoomExtentsModifier * zem = [[SCIZoomExtentsModifier alloc] init];
    [zem setModifierName:@"ZoomExtents Modifier"];
    
    SCIRolloverModifier * rollover = [[SCIRolloverModifier alloc] init];
    rollover.style.tooltipSize = CGSizeMake(200, NAN);
    [rollover setModifierName:@"Rollover Modifier"];
    
    SCIModifierGroup * gm = [[SCIModifierGroup alloc] initWithChildModifiers:@[xDragModifier, yDragModifier, pzm, zem, rollover]];
    surface.chartModifier = gm;
}

@end
